Enum syndicate::actor::LinkedTaskTermination [−][src]
pub enum LinkedTaskTermination {
Normal,
KeepFacet,
}Expand description
Linked tasks terminate yielding values of this type.
Variants
Normal
KeepFacet
Causes no action to be taken regarding the task’s associated Facet at the time the task returns.