Struct syndicate::actor::LoanedItem [−][src]
A LoanedItem<T> is a T with an associated cost recorded
against it in the ledger of a given Account. The cost is
repaid automatically when the LoanedItem<T> is Dropped.
LoanedItems are part of the flow control mechanism - see the
module-level documentation for more.
Fields
account: Arc<Account>The account against which this loan is recorded.
cost: usizeThe cost of this particular T.
item: TThe underlying item itself.
Implementations