pub struct RunningActor {
    pub actor_id: ActorId,
    pub root: FacetId,
    /* private fields */
}
State associated with each non-terminated Actor.

Fields

actor_id: ActorId

The ID of the actor this state belongs to.

root: FacetId

Implementations

source

impl RunningActor

source

pub fn shutdown(&self)

Requests a shutdown of the actor. The shutdown request is handled by the actor’s main loop, causing it to terminate with exit status Ok(()).

source

pub fn add_exit_hook<M: 'static + Send>(&mut self, r: &Arc<Ref<M>>)

Registers the entity r in the list of exit hooks for this actor. When the actor terminates, r’s exit_hook will be called in the context of the actor’s root facet.