Struct syndicate::skeleton::Index

source ·
pub struct Index { /* private fields */ }
Index of assertions and Observers.

Generally speaking, you will not need to use this structure; instead, create Dataspace entities.

Implementations

source

impl Index

source

pub fn new() -> Self

Construct a new Index.

source

pub fn add_observer( &mut self, t: &mut Activation, pat: &Pattern, observer: &Arc<Cap> )

Adds a new observer. If any existing assertions in the index match pat, establishes corresponding assertions at observer. Once the observer is registered, subsequent arriving assertions will be matched against pat and delivered to observer if they match.

source

pub fn remove_observer( &mut self, t: &mut Activation, pat: Pattern, observer: &Arc<Cap> )

Removes an existing observer.

source

pub fn insert(&mut self, t: &mut Activation, outer_value: &AnyValue) -> bool

Inserts an assertion into the index, notifying matching observers. Answers true iff the assertion was new to the dataspace.

source

pub fn remove(&mut self, t: &mut Activation, outer_value: &AnyValue) -> bool

Removes an assertion from the index, notifying matching observers. Answers true if it is the last in its equivalence class to be removed.

source

pub fn send(&mut self, t: &mut Activation, outer_value: &AnyValue)

Routes a message using the index, notifying matching observers.

source

pub fn assertion_count(&self) -> usize

Retrieves the current count of distinct assertions in the index.

source

pub fn endpoint_count(&self) -> isize

Retrieves the current count of assertions in the index, including duplicates.

source

pub fn observer_count(&self) -> usize

Retrieves the current count of observers of the index.