pub struct Ref<M> {
pub mailbox: Arc<Mailbox>,
pub facet_id: FacetId,
pub target: Mutex<Option<Box<dyn Entity<M>>>>,
}Expand description
A reference to an object that expects messages/assertions of type
M.
The object can be in the same actor, in a different local (in-process) actor, or accessible across a network link.
Fields
mailbox: Arc<Mailbox>Mailbox of the actor owning the referenced entity.
facet_id: FacetIdID of the facet (within the actor) owning the referenced entity.
target: Mutex<Option<Box<dyn Entity<M>>>>Mutex owning and guarding the state backing the referenced entity.
Implementations
sourceimpl<M> Ref<M>
impl<M> Ref<M>
sourcepub fn become_entity<E: 'static + Entity<M>>(&self, e: E)
pub fn become_entity<E: 'static + Entity<M>>(&self, e: E)
Supplies the behaviour (e) for a Ref created via
create_inert.
Panics
Panics if this Ref has already been given a behaviour.