pub struct RunningActor {
pub actor_id: ActorId,
pub root: FacetId,
/* private fields */
}
State associated with each non-terminated Actor.
The ID of the actor this state belongs to.
Requests a shutdown of the actor. The shutdown request is
handled by the actor’s main loop, causing it to terminate with
exit status Ok(()).
Registers the entity r in the list of exit hooks for this actor. When the actor
terminates, r’s exit_hook will be called in the context of the
actor’s root facet.
Immutably borrows from an owned value.
Read more
Mutably borrows from an owned value.
Read more
Returns the argument unchanged.
Instruments this type with the provided
Span, returning an
Instrumented wrapper.
Read more
Instruments this type with the provided
Span, returning an
Instrumented wrapper.
Read more
Instruments this type with the
current Span, returning an
Instrumented wrapper.
Read more
Calls U::from(self).
That is, this conversion is whatever the implementation of
From<T> for U chooses to do.
The type returned in the event of a conversion error.
Performs the conversion.
The type returned in the event of a conversion error.
Performs the conversion.