pub type AnyValue = ArcValue<Arc<Cap>>;Expand description
The type of messages and assertions that can be exchanged among distributed objects, including via dataspace.
A Preserves value where embedded references are instances of
Cap.
While Ref<M> can be used within a process, where arbitrary
M-values can be exchanged among objects, for distributed or
polyglot systems a lingua franca has to be chosen. AnyValue is
that language.